//------------------------------------------------------------------------------
// Includes                          
//------------------------------------------------------------------------------

    #include <windows.h>                        // Header pour les Applications Windows
    #include <stdlib.h>
    #include <stdio.h>                          // Header d'Entre/Sortie Standard
    #include <gl/glut.h>                        // Header OpenGL Utility Toolkit (GLUT)
    #include <string.h>                         // Header pour sprintf
    #include <time.h>

//------------------------------------------------------------------------------
// Constantes
//------------------------------------------------------------------------------

    #define EXIT {fclose(fichier);return -1;}   // ferme un fichier
    #define CTOI(C) (*(int*)&C)                 // rcupre en int un nombre point par un char*

//------------------------------------------------------------------------------
// Variables
//------------------------------------------------------------------------------

    double  a=0, b=0, z=5;                                // pour la rotation
    int     id_1=0, id_2=0;
    GLenum  TabBlend[9] = {GL_ZERO, GL_ONE, GL_DST_COLOR, GL_ONE_MINUS_DST_COLOR, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_DST_ALPHA, GL_ONE_MINUS_DST_ALPHA, GL_SRC_ALPHA_SATURATE};
    char    title[30];
    GLuint  Name[50];
    GLUquadricObj*  g_quadratic;
    
/*  // Pour les lumires
    float MatSpec[4] = {1.f,1.f,1.f,1.f};
    float MatDif[4] = {.8f,.8f,.5f,1.f};
    float MatAmb[4] = {.8f,.8f,.5f,1.f};
    int   LightPos[4] = {0,0,2,0};
    float LightDif[4] = {.6f,1.f,1.f,1.f};
    float LightSpec[4] = {.6f,1.f,1.f,1.f};
    float LightAmb[4] = {.1f,.1f,.1f,1.f};
*/

//------------------------------------------------------------------------------
// Charge une image TGA 32 bits non compresse
//------------------------------------------------------------------------------

int    LoadTGA(char *filename, int nb) // Loads A TGA File Into Memory
 {    
    GLubyte TGAheader[12]={0,0,2,0,0,0,0,0,0,0,0,0};// Uncompressed TGA Header
    GLubyte TGAcompare[12];                         // Used To Compare TGA Header
    GLubyte header[6];                              // First 6 Useful Bytes From The Header
    int     imageSize;                              // Used To Store The Image Size When Setting Aside Ram
    int     type;                                   // Set The Default GL Mode To RBGA (32 BPP)
    GLubyte *imageData;                             // Donnes de l'image, jusqu' 32 bits
    int     bpp;                                    // Bits Par Pixel de l'image
    int     Width, Height;                          // Taille de l'image
                                                    
    // Lit le fichier et son header
    FILE *fichier = fopen(filename, "rb");          // Open The TGA File                                                    
    if (fread(TGAcompare,1,sizeof(TGAcompare),fichier)!=sizeof(TGAcompare)) EXIT; // Are There 12 Bytes To Read?
    if (memcmp(TGAheader,TGAcompare,sizeof(TGAheader))!=0)                  EXIT; // Does The Header Match What We Want?
    if (fread(header,1,sizeof(header),fichier)!=sizeof(header))             EXIT; // If So Read Next 6 Header Bytes

    // Rcupre les infos de l'image
    Width  = header[1] * 256 + header[0];           // Determine The TGA Width    (highbyte*256+lowbyte)
    Height = header[3] * 256 + header[2];           // Determine The TGA Height    (highbyte*256+lowbyte)
    bpp = header[4];                                // Grab The TGA's Bits Per Pixel (24 or 32)
    if (bpp==24) type=GL_RGB;                       // If So Set The 'type' To GL_RGB
    else type=GL_RGBA;                              // If So Set The 'type' To GL_RGBA
    imageSize = Width*Height*bpp/8;                 // Calculate The Memory Required For The TGA Data

    // Charge l'image
    imageData=(GLubyte *)malloc(imageSize);         // Reserve Memory To Hold The TGA Data
    if (fread(imageData, 1, imageSize, fichier)!=imageSize) // Does The Image Size Match The Memory Reserved?
     {
        free ( imageData );
        EXIT;
     }

    fclose (fichier);                               // Close The File
    
    // Inverse R et B
    int     t, i;
    for(i=0; i<imageSize;i+=bpp/8)              // Loop Through The Image Data
     {                                              // Swaps The 1st And 3rd Bytes ('R'ed and 'B'lue)
        t=imageData[i];                             // Temporarily Store The Value At Image Data 'i'
        imageData[i]=imageData[i+2];                // Set The 1st Byte To The Value Of The 3rd Byte
        imageData[i+2]=t;                           // Set The 3rd Byte To The Value In 'temp' (1st Byte Value)
     }

    // Build A Texture From The Data
    glPixelStorei(GL_UNPACK_ALIGNMENT,1);
    glGenTextures(nb, &Name[nb]);                   // Generate OpenGL texture IDs                                             
    glBindTexture(GL_TEXTURE_2D, Name[nb]);         // Bind Our Texture
    
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);

    glTexImage2D(GL_TEXTURE_2D, 0, type, Width, Height, 0, type, GL_UNSIGNED_BYTE, imageData);
    // target, mipmap lvl, nb coul, larg, haut, large bord, type coul, code compo, img)

    return Name[nb];
 }

//------------------------------------------------------------------------------
// LoadBMP : charge une image 24bpp
//------------------------------------------------------------------------------

int LoadBMP(char *File, int nb)
 {
    unsigned char   *Data;
    FILE            *fichier;
    unsigned char   Header[0x36];
    GLuint          DataPos,DataSize;
    GLint           Components;
    GLsizei         Width,Height;
    GLenum          Format,Type;
    
    // Lit le fichier et son header, puis fait diffrents tests d'authenticit
    fichier = fopen(File,"rb");
    if (!fichier) return -1;
    if (fread(Header,1,0x36,fichier)!=0x36) EXIT;
    if (Header[0]!='B' || Header[1]!='M')   EXIT;
    if (CTOI(Header[0x1E])!=0)              EXIT;
    if (CTOI(Header[0x1C])!=24)             EXIT;

    // Rcupre les infos de l'image
    Width  = CTOI(Header[0x12]);
    Height = CTOI(Header[0x16]);    
    Type   = GL_UNSIGNED_BYTE;
    Format = GL_RGB;
    Components = 3;
    
    // Rcupre les infos du fichier
    DataPos  = CTOI(Header[0x0A]);
    DataSize = CTOI(Header[0x22]);
    if (DataPos==0)  DataPos=0x36;
    if (DataSize==0) DataSize=Width*Height*Components;
    
    // Charge l'image
    fseek(fichier,DataPos,0);
    Data = ( unsigned char * ) malloc ( DataSize * sizeof ( unsigned char ));
    if (!Data)  EXIT;

    if (fread(Data,1,DataSize,fichier)!=DataSize) 
     {
        free ( Data );
        EXIT;
     }
    
    fclose(fichier);

    // Inverse R et B (BVR->RVB)
    unsigned char t;
    int x;
    for (x=0;x<Width*Height;x++)
     {
        t=Data[x*3];
        Data[x*3]=Data[x*3+2];
        Data[x*3+2]=t;
     }

    // Envoie la texture  OpenGL
    glPixelStorei(GL_UNPACK_ALIGNMENT,1);
    glGenTextures(nb, &Name[nb]);
    glBindTexture(GL_TEXTURE_2D, Name[nb]);

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);

    glTexImage2D(GL_TEXTURE_2D, 0, Components, Width, Height, 0, Format, Type, Data); 
    // target, mipmap lvl, nb coul, larg, haut, large bord, type coul, code compo, img)

    return Name[nb];
 }

//------------------------------------------------------------------------------
// Fonction de redimensionnement
//------------------------------------------------------------------------------

void    Reshape(int w,int h)
 {
    glViewport(0,0,w,h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45,(float)(w)/(float)(h),1,100);
 }

//------------------------------------------------------------------------------
// Fonction d'initialisation
//------------------------------------------------------------------------------

void    InitGL()
 {
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);           // Arrire Plan Noir
    glClearDepth(1.0f);                             // Configuration de la profondeur du buffer
     
//    glBlendFunc(TabBlend[id_1], TabBlend[id_2]);
//    glBlendFunc(TabBlend[1], TabBlend[4]);
//    glEnable(GL_BLEND);

    glDepthFunc(GL_LEQUAL);                         // Le type de profondeur  tester
    glEnable(GL_DEPTH_TEST);                        // Active le test de profondeur, peut-tre pour corriger des imperfections

    glAlphaFunc(GL_GREATER,0.1f);                   // Permet de choisir prsicemment un alpha sur les TGA
    glEnable(GL_ALPHA_TEST);                        // Active l'alpha

    glEnable(GL_TEXTURE_2D);                        // Textures 2D actives

    // pour les sphres
    g_quadratic=gluNewQuadric();
	gluQuadricNormals(g_quadratic, GLU_SMOOTH);
	gluQuadricTexture(g_quadratic, GL_TRUE);

/*    LUMIERE
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,MatSpec);
	glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,MatDif);
	glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,MatAmb);
	glMateriali(GL_FRONT_AND_BACK,GL_SHININESS,50);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,LightDif);
	glLightfv(GL_LIGHT0,GL_SPECULAR,LightSpec);
	glLightfv(GL_LIGHT0,GL_AMBIENT,LightAmb);	

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
*/
 }

//------------------------------------------------------------------------------
// Gestion des touches standard
//------------------------------------------------------------------------------

void    GestionClavier(unsigned char key, int x, int y)  
 {  
    switch (key)
     {
        case 32 :
            id_2++;
            if (id_2 == 9) { id_2 = 0; id_1++; }
            if (id_1 == 8) { id_2 = 0; id_1 = 0; }
            sprintf(title, "Fentre : %d - %d", id_1, id_2);
            glutSetWindowTitle(title);
            InitGL();
        break;
     }
 }

//------------------------------------------------------------------------------
// Fonction de dessin
//------------------------------------------------------------------------------

void    Draw()
 {
    static float fire = 0;
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glMatrixMode(GL_MODELVIEW);

	a+=1;
    b+=2;

/*  // lumire
	glLoadIdentity();	
	glTranslated(0,0,-z);
	glRotated(a,0,1,0);
	glLightiv(GL_LIGHT0,GL_POSITION,LightPos);
*/
	
    // sphre 1
	glLoadIdentity();
	glTranslated(0,0,-z);
	glRotated(fire+=rand()%10,0,1,0);
    glBindTexture(GL_TEXTURE_2D, Name[1]);
	gluSphere(g_quadratic,1,25,25);

/*		
    // cube
	glLoadIdentity();
	gluLookAt(3,2,3,0,0,0,0,1,0);
	glRotated(a,0,1,0);
    
    glBegin(GL_QUADS);

        glTexCoord2i(0,0);glVertex3i(-1,-1,-1);
        glTexCoord2i(1,0);glVertex3i(+1,-1,-1);
        glTexCoord2i(1,1);glVertex3i(+1,+1,-1);
        glTexCoord2i(0,1);glVertex3i(-1,+1,-1);

        glTexCoord2i(0,0);glVertex3i(-1,-1,+1);
        glTexCoord2i(1,0);glVertex3i(+1,-1,+1);
        glTexCoord2i(1,1);glVertex3i(+1,+1,+1);
        glTexCoord2i(0,1);glVertex3i(-1,+1,+1);

        glTexCoord2i(0,0);glVertex3i(+1,-1,-1);
        glTexCoord2i(1,0);glVertex3i(+1,-1,+1);
        glTexCoord2i(1,1);glVertex3i(+1,+1,+1);
        glTexCoord2i(0,1);glVertex3i(+1,+1,-1);

        glTexCoord2i(0,0);glVertex3i(-1,-1,-1);
        glTexCoord2i(1,0);glVertex3i(-1,-1,+1);
        glTexCoord2i(1,1);glVertex3i(-1,+1,+1);
        glTexCoord2i(0,1);glVertex3i(-1,+1,-1);

        glTexCoord2i(0,0);glVertex3i(-1,+1,-1);
        glTexCoord2i(1,0);glVertex3i(+1,+1,-1);
        glTexCoord2i(1,1);glVertex3i(+1,+1,+1);
        glTexCoord2i(0,1);glVertex3i(-1,+1,+1);

        glTexCoord2i(0,0);glVertex3i(-1,-1,+1);
        glTexCoord2i(1,0);glVertex3i(+1,-1,+1);
        glTexCoord2i(1,1);glVertex3i(+1,-1,-1);
        glTexCoord2i(0,1);glVertex3i(-1,-1,-1);

    glEnd();
*/
    glutSwapBuffers();
    glutPostRedisplay();
    
 }

//------------------------------------------------------------------------------
// Fonction Principale
//------------------------------------------------------------------------------

int main( int argc, char *argv[ ], char *envp[ ] )
 {
    srand ( time ( NULL ) );
    glutInit(&argc,argv);
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
    sprintf(title, "Fire : %d - %d", id_1, id_2);
    glutCreateWindow(title);

    LoadTGA("fireboule.tga", 1);
    LoadTGA("fireboule2.tga", 2);
    glutReshapeFunc(Reshape);
    glutDisplayFunc(Draw);
    glutKeyboardFunc(GestionClavier);
    InitGL();
    glutMainLoop();

    return 0;
 }

//------------------------------------------------------------------------------
// THE END
//------------------------------------------------------------------------------
