//------------------------------------------------------------------------------
// Includes
//------------------------------------------------------------------------------

    #include <windows.h>                            // Header pour les Applications Windows
    #include <stdlib.h>                             // Header pour le random
    #include <gl/glut.h>                            // Header OpenGL Utility Toolkit (GLUT)
    #include <math.h>                               // Header mathmatique

//------------------------------------------------------------------------------
// Constantes & Variables
//------------------------------------------------------------------------------

    #define pi          3.1415926535897932384626433832795f // PI pour les angles
    #define maxp        5000                          // nombre de points
    #define ARC_EN_CIEL glColor3f(1.0f*float(cos(arc_en_ciel/20.0f)),1.0f*float(sin(arc_en_ciel/25.0f)),1.0f-0.5f*float(cos(arc_en_ciel/17.0f)));

    struct points
     {
        float StartX;
        float StartY;
        float StartZ;
        float InitXVel;
        float InitYVel;
        float InitZVel;
        float wind;
        float color[3]; // Les 3 couleurs RVB
        float intensity;
        float t;
     }  point[maxp];

    int    i;
    double a=0;    //Angles
    float  arc_en_ciel = 0.0f;                     // Utilise pour une couleur arc-en-ciel dans le menu

//------------------------------------------------------------------------------
// Fonction de redimensionnement
//------------------------------------------------------------------------------

void Reshape(int width, int height)
{
    glPopMatrix();
    glViewport(0,0,width,height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(120,float(width)/float(height),0.01,200);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
}

//------------------------------------------------------------------------------
// Fonction d'initialisation
//------------------------------------------------------------------------------

void InitGL()
{
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);           // Arrire Plan Noir
    glClearDepth(1.0f);                             // Configuration de la profondeur du buffer
    glEnable(GL_COLOR);

    glDepthFunc(GL_LEQUAL);                         // Le type de profondeur  tester
    glEnable(GL_DEPTH_TEST);                        // Active le test de profondeur, peut-tre pour corriger des imperfections

    glBlendFunc(GL_SRC_ALPHA, GL_ONE);              // Notre fonction de blending (lorsqu'il sera activ par la touche B)
    glEnable(GL_BLEND);                            // Dsactive le blending (effet de mlange de couleur, activ avec la touche B)

    glAlphaFunc(GL_GREATER,0.5f);                   // Permet de choisir prsicemment un alpha sur les TGA
    glEnable(GL_ALPHA_TEST);                        // Active l'alpha

    glShadeModel(GL_SMOOTH);                        // Smooth Shading Activ, il doit lisser les formes (?)
    glColorMaterial(GL_FRONT_AND_BACK,GL_SPECULAR);//Ca marche aussi avec specular !

    glPointSize(3);
}

//------------------------------------------------------------------------------
// Retourne le temps coul (FPS ?)
//------------------------------------------------------------------------------

int    temps()
 {
    static int  time;
    static int  timebase=0;
    static int  tempnb=0;
    int temp;

    time = glutGet(GLUT_ELAPSED_TIME);
    temp = timebase;
    timebase = time;

    return time - temp;
 }

//------------------------------------------------------------------------------
// Initialisation des points
//------------------------------------------------------------------------------

void InitPoint(points &ipoint)
 {
    float vel=5;
    float angle=(rand()*30+30)/float(180*pi);

    ipoint.StartX = cos(angle);
    ipoint.StartY = sin(angle);
    ipoint.StartZ = cos(angle);

    ipoint.InitXVel = ipoint.StartX*vel;
    ipoint.InitYVel = ipoint.StartY*vel;
    ipoint.InitZVel = ipoint.StartZ*vel;

    ipoint.wind = 25;

    ipoint.color[0] = 1.0f*cos(arc_en_ciel*0.05);
    ipoint.color[1] = 1.0f*sin(arc_en_ciel*0.04);
    ipoint.color[2] = 1.0f-0.5f*cos(arc_en_ciel*0.0588);

    ipoint.intensity = 1;
    ipoint.t = 0;
 }

//------------------------------------------------------------------------------
// Fonction de dessin
//------------------------------------------------------------------------------

void Draw()
{
    float   x=0, y=0, z=0;
    float   addtemps=temps()/float(2000);

    a+=0.5;
    arc_en_ciel+= 0.5f;                             // Pour la couleur arc-en-ciel

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    gluLookAt(0,0,200,0,0,0,0,1,0);
    glRotatef(a,0,0,1);
    
    glLineWidth(rand()*0.001);

    for (i=0; i<maxp; i++)
     {
        glBegin(GL_LINES);
            x = point[i].StartX;
            y = point[i].StartY + point[i].InitYVel * point[i].t;
            z = point[i].StartZ + point[i].InitZVel * point[i].t + point[i].wind * pow(point[i].t,2);
            glColor3f(point[i].intensity*point[i].color[0],point[i].intensity*point[i].color[1],point[i].intensity*point[i].color[2]);
            glVertex3f(x, y, z);

            x = point[i].StartX;
            y = point[i].StartY + point[i].InitYVel * (point[i].t-15*0.0005);
            z = point[i].StartZ + point[i].InitZVel * (point[i].t-15*0.0005) + point[i].wind * pow((point[i].t-15*0.0005),2);
            glColor3f(0,0,0);
            glVertex3f(x, y, z);
        glEnd();

        point[i].t+=addtemps;
        point[i].intensity-=0.001*point[i].t;
        
        if ((point[i].intensity<0) || (y>10) || (y<-10) || (x>10) || (x<-10)) InitPoint(point[i]);
     }

    glutSwapBuffers();
    glutPostRedisplay();
 }

//------------------------------------------------------------------------------
// Fonction Principale
//------------------------------------------------------------------------------

int main( int argc, char *argv[ ], char *envp[ ] )
{
    glutInit(&argc,argv);
    glutInitWindowSize(300,300);
    glutInitWindowPosition (100,100);
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
    glutCreateWindow("Effet (release 9)");

    void randomize(void);
    for (i=0; i<maxp; i++)
        InitPoint(point[i]);

    InitGL();
    glutReshapeFunc(Reshape);
    glutDisplayFunc(Draw);
    glutMainLoop();

    return 0;
}

//------------------------------------------------------------------------------
// THE END
//------------------------------------------------------------------------------

