//------------------------------------------------------------------------------
// Includes
//------------------------------------------------------------------------------

    #include <windows.h>                            // Header pour les Applications Windows
    #include <stdlib.h>                             // Header pour le random
    #include <gl/glut.h>                            // Header OpenGL Utility Toolkit (GLUT)
    #include <math.h>                               // Header mathmatique

//------------------------------------------------------------------------------
// Constantes & Variables
//------------------------------------------------------------------------------

    #define pi          3.1415926535897932384626433832795f // PI pour les angles
    #define maxp        1000                          // nombre de points
    #define ARC_EN_CIEL glColor3f(1.0f*float(cos(arc_en_ciel/20.0f)),1.0f*float(sin(arc_en_ciel/25.0f)),1.0f-0.5f*float(cos(arc_en_ciel/17.0f)));

    struct points
     {
        float StartX;
        float StartY;
        float StartZ;
        float InitXVel;
        float InitYVel;
        float InitZVel;
        float wind;
        float color[3]; // Les 3 couleurs RVB
        float intensity;
        float t;
     }  point[maxp];

    int    i;
    float  arc_en_ciel = 0.0f;                     // Utilise pour une couleur arc-en-ciel dans le menu

//------------------------------------------------------------------------------
// Fonction de redimensionnement
//------------------------------------------------------------------------------

void Reshape(int width, int height)
{
    glPopMatrix();
    glViewport(0,0,width,height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45,float(width)/float(height),0.01,100);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
}

//------------------------------------------------------------------------------
// Fonction d'initialisation
//------------------------------------------------------------------------------

void InitGL()
{
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);           // Arrire Plan Noir
    glClearDepth(1.0f);                             // Configuration de la profondeur du buffer
    glEnable(GL_COLOR);

    glPointSize(3);
}

//------------------------------------------------------------------------------
// Retourne le temps coul (FPS ?)
//------------------------------------------------------------------------------

int    temps()
 {
    static int  time;
    static int  timebase=0;
    static int  tempnb=0;
    int temp;

    time = glutGet(GLUT_ELAPSED_TIME);
    temp = timebase;
    timebase = time;

    return time - temp;
 }

//------------------------------------------------------------------------------
// Initialisation des points
//------------------------------------------------------------------------------

void InitPoint(points &ipoint)
 {
    float vel=15;
    float angle=(rand())/float(180*pi);

    ipoint.StartX = cos(angle);
    ipoint.StartY = sin(angle);
    ipoint.StartZ = cos(angle)*sin(angle);

    ipoint.InitXVel = ipoint.StartX*vel;
    ipoint.InitYVel = ipoint.StartY*vel;
    ipoint.InitZVel = ipoint.StartZ*vel;
    
    ipoint.StartX = ipoint.StartY = ipoint.StartZ = 0;

    ipoint.wind = 15+rand()%5;

    ipoint.color[0] = 1.0f*cos(arc_en_ciel*0.05);
    ipoint.color[1] = 1.0f*sin(arc_en_ciel*0.04);
    ipoint.color[2] = 1.0f-0.5f*cos(arc_en_ciel*0.0588);

    ipoint.intensity = 1;
    ipoint.t = 0;
 }

//------------------------------------------------------------------------------
// Fonction de dessin
//------------------------------------------------------------------------------

void Draw()
{
    float   x=0, y=0;
    float   gravity = 9.8;
    float   addtemps=temps()/float(2000);

    arc_en_ciel+= 0.5f;                             // Pour la couleur arc-en-ciel

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    gluLookAt(0,0,20,0,0,0,0,1,0);
    
    glBegin(GL_POINTS);
    for (i=0; i<maxp; i++)
     {
        x = point[i].StartX + point[i].InitXVel * point[i].t + point[i].wind * pow(point[i].t,2);
        y = point[i].StartY + point[i].InitYVel * point[i].t - gravity * pow(point[i].t,2);
        glColor3f(point[i].intensity*point[i].color[0],point[i].intensity*point[i].color[1],point[i].intensity*point[i].color[2]);
        glVertex3f(x, y, 0);

        point[i].t+=addtemps;
        point[i].intensity-=0.01*point[i].t;
        if ((point[i].intensity<0) || (y<-10) || (x>10)) InitPoint(point[i]);
     }
    glEnd();

    glutSwapBuffers();
    glutPostRedisplay();
 }

//------------------------------------------------------------------------------
// Fonction Principale
//------------------------------------------------------------------------------

int main( int argc, char *argv[ ], char *envp[ ] )
{
    glutInit(&argc,argv);
    glutInitWindowSize(300,300);
    glutInitWindowPosition (100,100);
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
    glutCreateWindow("Effet (release 14)");

    void randomize(void);
    for (i=0; i<maxp; i++)
        InitPoint(point[i]);

    InitGL();
    glutReshapeFunc(Reshape);
    glutDisplayFunc(Draw);
    glutMainLoop();

    return 0;
}

//------------------------------------------------------------------------------
// THE END
//------------------------------------------------------------------------------

